/*
 $Revision: 1.34.2.17 $
 $Date: 2005/01/19 13:36:39 $
*/
#include "bookreader.h"

#include <new>

#include <qwidget.h>
#include <qpainter.h>
#include <qpe/qpeapplication.h>
#include <qfile.h>
#include <qtextstream.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "loadpng.h"
#include "loadjpeg.h"
//#include "loadgif.h"
#include "imagedata.h"


/* 
 *  Constructs a Example which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
BookReader::BookReader( const BR_Option opt)
  : QWidget( 0, "BookReader", WStyle_Customize | WStyle_NoBorderEx),
    option(opt)
{

  brControl = 0;
  my_has_fullscreen = false;
  showScreen = true;
  setBackgroundMode(NoBackground);

  option.keepScale = false;

  fileListIt = option.filelist->find(option.filename); // first image

  

  //
  // initialize nibble mode
  //
  switch (option.nib.mode) {
  case NIB_AutoHoriz:
    option.autoscale = AS_Height;
    option.nib.divV = 1;           // Delete Vertical divider 
  case NIB_Horiz:
    horizPrimary = true;
    break;

  case NIB_AutoVert:
    option.autoscale = AS_Width;
    option.nib.divH = 1;           // Delete Horizonal divider 
  case NIB_Vert:
    horizPrimary = false;
    break;

  case NIB_None:
  default:
    horizPrimary = false;          // Standerd Vertical nibble mode
    option.nib.divH = 1;           // Delete Horizonal divider 
    option.nib.divV = 1;           // Delete Vertical divider 
    break;
  }

  nibMax[true] = option.nib.divH;  // Horizonal divider
  nibMax[false] = option.nib.divV; // Vertical divider

  my_nib[true] = 1;                // Current Horizonal posision for nibble mode
  my_nib[false] = 1;               // Current Vertical posision for nibble mode



  //
  // save standard option
  //
  org_ratio = option.ratio;
  org_centX = option.cent_X;
  org_centY = option.cent_Y;
  org_autoscale = option.autoscale;



  //
  // set initial rotation mode
  //
  if(option.rot){
    setMinimumSize(640,480);
    imageRotation = true;
    option.cent_X = org_centX; option.cent_Y = org_centY;
  }else{
    setMinimumSize(480,640);
    imageRotation = false;
    option.cent_X = org_centY; option.cent_Y = org_centX;
  }



  //
  // keyboard grab
  //
  //  QPEApplication::grabKeyboard();
  //  qApp->processEvents();

  option.ratio = loadFileSet( org_ratio  );

}

/*  
 *  Destroys the object and frees any allocated resources
 */

BookReader::~BookReader()
{

}


bool BookReader::event(QEvent *e)
{

  if(e->type() == 25){
    showScreen = false;
    return false;
  }
    
  return QWidget::event(e);
}


void BookReader::closeEvent(QCloseEvent *)
{
  //
  // keyboard ungrab
  //
  //  QPEApplication::grabKeyboard();
  //  qApp->processEvents();
  //  QPEApplication::ungrabKeyboard();
  //  qApp->processEvents();

  if(option.auto_bookmark)
    makeBookmark();
  emit closed();
}

void BookReader::makeBookmark()
{
  //
  //make bookmark file and save filename
  //
  QFile f(option.dir_name+"/.bookmark");
  if ( f.open(IO_ReadWrite | IO_Truncate) ){
    QTextStream ts(&f);
    ts << *fileListIt;
    f.close();
  }
}


void BookReader::focusInEvent(QFocusEvent *)
{
  // Always do it here, no matter the size.
      enableFullscreen();
}

void BookReader::paintEvent( QPaintEvent * e)
{

  //  QFocusEvent *fe;
  //  fe = new QFocusEvent(QEvent::FocusIn);
  //  QApplication::sendEvent(this, fe);
  //  my_has_fullscreen = false;
  //  enableFullscreen();

  //  setWFlags(WStyle_Customize | WStyle_NoBorderEx);
  //    setMinimumSize(640,480);

  //  showNormal();
  //  showFullScreen();
  if(showScreen == false )
    return;

  if( e->rect().height() < 400 )
    return;


  my_painter = new QDirectPainter(this);
  short *fb = (short*)my_painter->frameBuffer();
  short *src = (short*)paintImage.bits();
  

  if( brControl !=0){

    int brLeft;
    int brRight;
    int brTop;
    int brBottom;

    int bottom;
    int right;
    
    //
    //calculate control widget size with border line
    //Not paint taskbar.
    //
    if(option.rot){
      brLeft = brControl->geometry().left()-5;
      brRight = brControl->geometry().right()+5;
      brTop = brControl->geometry().top()-34;
      brBottom = brControl->geometry().bottom()+6;
      bottom = 38; right = 640;
    }else{
      brBottom = 480-brControl->geometry().left()+5;
      brTop = 480-brControl->geometry().right()-5;
      brLeft = brControl->geometry().top()-34;
      brRight = brControl->geometry().bottom()+6;
      bottom = 0; right = 602;
    }

    //
    // repaint using direct painter for showing controler
    // and taskbar
    //
    for(int y=0;y <640; y++){
      for(int x=0;x <480; x++){
	if ( (y > brLeft) && (y < brRight) && 
	     (x > 480-brBottom) &&(x < 480-brTop) ||
	     (x < bottom ) || (y > right)){
	  fb++;
	  src++;
	}else{
	  *fb++=*src++;
	}
      }
    }
  }else{

    //
    // repaint full screen
    //
    for(int i=0;i <640*480; i++){
      *fb++=*src++;
    }
  }

  delete my_painter;
  my_painter=0;
  
}

void BookReader::mousePressEvent( QMouseEvent *e)
{
  //
  // memory press point
  //
  if( (option.rot ^ imageRotation) ){
    prev_x = e->globalY();
    prev_y = e->globalX();
  }else{
    prev_x = e->globalX();
    prev_y = e->globalY();
  }
  
}

void BookReader::mouseReleaseEvent( QMouseEvent *e)
{

  //
  // select file area check
  //
  if (option.rot && (e->globalY() >400)){
    if(e->globalX() >320){
      prevImage();
    }else{
      nextImage();
    }
  }else if( (!option.rot) && (e->globalY() >550)){
    if(e->globalX() >240){
      prevImage();
    }else{
      nextImage();
    }
  }else{

    //
    // calculate mouse move posision 
    //
    int tmp_x;
    int tmp_y;
    if( (option.rot ^ imageRotation) ){
      if (option.rot){
      tmp_x = prev_x - e->globalY();
      tmp_y = e->globalX() - prev_y;
      }else{
      tmp_x = e->globalY() - prev_x;
      tmp_y = prev_y - e->globalX();
      }
    }else{
      tmp_x = e->globalX() - prev_x;
      tmp_y = e->globalY() - prev_y;
    }
    if( (((tmp_x * tmp_x) + (tmp_y * tmp_y)) > 20) && (brControl == 0) ){
      option.cent_X += tmp_x;
      option.cent_Y += tmp_y;
      if(imageRotation)
	makePaintImageR();
      else
	makePaintImage();
      repaint();
    }
  }

}

void BookReader::mouseDoubleClickEvent( QMouseEvent *e)
{
  if ( (option.rot && (e->globalY() <400)) || ((!option.rot) && (e->globalY() >550)) ){
    brControlCreate();
  }
}

void BookReader::brControlCreate()
{

  //
  // show controler
  //
  if (brControl == 0){
    brControl = new BRControl(option, this);
    connect(brControl, SIGNAL(apply()), this, SLOT(testView()) ); 
    connect(brControl, SIGNAL(bookmark()), this, SLOT(makeBookmark()) );
    brControl->show();

    int ok = brControl->exec();
    if (ok == 999){
      delete brControl;
      brControl = 0;
      close();
      return;
    }
    option.cent_X = brControl->getCenter_X(ok);
    option.cent_Y = brControl->getCenter_Y(ok);
    option.bgcolor = brControl->getBGColor(ok);
    option.ratio = brControl->getScale(ok);
    option.autoscale = brControl->getAutoScale(ok);
    option.keepScale = brControl->getKeepScale(ok);

    delete brControl;
    brControl = 0;
    my_has_fullscreen = false;
    showScreen = true;
    enableFullscreen();
    option.ratio = loadFileSet(option.ratio);
    makePaintImage();
    
  }
}

void BookReader::testView()
{
  //
  // contorler test view mode
  //
  if (brControl != 0){
    option.cent_X = brControl->getCenter_X(true);
    option.cent_Y = brControl->getCenter_Y(true);
    option.bgcolor = brControl->getBGColor(true);
    option.ratio = brControl->getScale(true);
    option.autoscale = brControl->getAutoScale(true);
    option.keepScale = brControl->getKeepScale(true);

    option.ratio = loadFileSet(option.ratio);
    makePaintImage();
    brControl->setScale(option.ratio);

  }
}


void BookReader::QueueKey(QKeyEvent *e, int pressed)
{

  //
  // cover close event
  //
  if(e->key() == 0x2009){
    close();
    return;
  }

  //
  //normal key event
  //
  if (pressed){
    switch(e->key()){
    case Key_Escape:
      close();
      return;
      break;
    case Key_R:
      rotationImage();
      break;
    case Key_Q:
      option.ratio+=5;
      if(option.ratio >2000)
	option.ratio = 2000;
      option.autoscale = AS_None;
      option.ratio = loadFileSet(option.ratio);
      makePaintImage();
      break;
    case Key_W:
      option.ratio-=5;
      if(option.ratio < 10)
	option.ratio = 10;
      option.autoscale = AS_None;
      option.ratio = loadFileSet(option.ratio);
      makePaintImage();
      break;
    case Key_A:
      option.autoscale = AS_Both;
      option.ratio = loadFileSet(100);
      
      makePaintImage();
      break;
    case Key_Up:
      prevImage();
      break;
    case Key_Down:
      nextImage();
      break;
    case Key_F11:               // Menu key
      brControlCreate();

    default:
      break;
    }
  }
  
}

void BookReader::rotationImage()
{
  
  //
  //rotation image and convert center XY
  //
  int tmp_swap;
  if (imageRotation){
    imageRotation = FALSE;
    tmp_swap = option.cent_X; 
    option.cent_X = 480- option.cent_Y;
    option.cent_Y = tmp_swap;
  }else{
    imageRotation = TRUE;
    tmp_swap = option.cent_X; 
    option.cent_X = option.cent_Y; 
    option.cent_Y = 480 - tmp_swap;
  }
  
  makePaintImage();
  
}

//
// select next file and view image
//
void BookReader::nextImage()
{
  //
  // move nibble posision
  //
  my_nib[horizPrimary]++;
  if(my_nib[horizPrimary] > nibMax[horizPrimary]){
    my_nib[horizPrimary] = 1;
    my_nib[!horizPrimary]++;
    if(my_nib[!horizPrimary] > nibMax[!horizPrimary]){
      my_nib[!horizPrimary] = 1;

      //
      // move to next file 
      //
      fileListIt++;
      if ( fileListIt == option.filelist->end()){
	fileListIt--;
      }else{
	if(!option.keepScale){
	  imageRotation = option.rot;
	  option.autoscale = org_autoscale;
	  option.ratio = org_ratio;
	}

	if(imageRotation == true){
	  option.cent_X = org_centX; option.cent_Y = org_centY;
	}else{
	  option.cent_X = org_centY; option.cent_Y= org_centX;
	}
	option.ratio = loadFileSet( option.ratio );
	
      }
      return;
    }
  }
  makePaintImage();
}

//
// select previous file and view image
//
void BookReader::prevImage()
{
  // move nibble posision
  if(option.nib.novel && (my_nib[horizPrimary] > 1))
    my_nib[horizPrimary] = 2;
  my_nib[horizPrimary]--;
  if(my_nib[horizPrimary] <= 0){
    my_nib[horizPrimary] = nibMax[horizPrimary];
    my_nib[!horizPrimary]--;
    if(my_nib[!horizPrimary] <= 0){
      my_nib[!horizPrimary] = nibMax[!horizPrimary];

      //
      // move to previous file
      //
      if ( fileListIt != option.filelist->begin()){
	fileListIt--;
	if(!option.keepScale){
	  imageRotation = option.rot;
	  option.autoscale = org_autoscale;
	  option.ratio = org_ratio;
	}
	  if(imageRotation == true){
	    option.cent_X = org_centX; option.cent_Y = org_centY;
	  }else{
	    option.cent_X = org_centY; option.cent_Y = org_centX;
	  }
	  option.ratio = loadFileSet( option.ratio );
      }
      return;
    }
  }
  makePaintImage();
}

void BookReader::changeStyle()
{
  option.rot = !option.rot;

  if(option.rot){
    setMinimumSize(640,480);
    //    imageRotation = true;
    //    option.cent_X = org_centX; option.cent_Y = org_centY;
  }else{
    setMinimumSize(480,640);
    //    imageRotation = false;
    //    option.cent_X = org_centY; option.cent_Y = org_centX;
  }

}

int BookReader::loadFileSet(int my_ratio)
{
  //
  //  load selected jpeg file and view
  //
  my_ratio = loadFile( (option.dir_name +"/"+ *fileListIt).utf8(),
			       my_ratio );

  int tmpWidth = (option.rot ? 640 : 480);
  int tmpHeight = (option.rot ? 480 : 640);

  //
  // auto nibble mode setting
  //
  switch ( option.nib.mode){
  case NIB_AutoVert:
    if ((scaledImage.height() - tmpHeight) <= 0){
      nibMax[false] = 1;
    }else{
      nibMax[false] = ((scaledImage.height() - tmpHeight) / (tmpHeight / 2)) + 2;
    }
    
    break;
  case NIB_AutoHoriz:
    if ((scaledImage.width() - tmpWidth) <= 0){
      nibMax[true] = 1;
    }else{
      nibMax[true] = ((scaledImage.width() - tmpWidth) / (tmpWidth / 2)) + 2;
    }

    break;
  default:
    break;
  }
  my_nib[true] = 1;
  my_nib[false] = 1;

  makePaintImage();
  return my_ratio;
}

//
// make image for paint event
//
void BookReader::makePaintImage()
{
  int h = scaledImage.height();
  int w = scaledImage.width();
  int nibH;

  //
  // nibble mode paint image center setting 
  //
  switch ( option.nib.mode){
  case NIB_Vert:
  case NIB_AutoVert:
  case NIB_Horiz:
  case NIB_AutoHoriz:
    if(option.nib.left)
      nibH = nibMax[true] - my_nib[true] + 1;
    else
      nibH = my_nib[true];

    if(option.rot){
      if ( (nibMax[false] != 1) && (h > 480) ){
	option.cent_Y = 240 + (h - 480) / 2 -
	  (h - 480) * (my_nib[false] - 1)/(nibMax[false] - 1);
	
      }
      if ( (nibMax[true] != 1) && (w > 640) ){
	option.cent_X = 320 - (w - 640) /2 +
	  (w - 640) * (nibH - 1)/(nibMax[true] - 1);
    }
    }else{
      if ( (nibMax[false] != 1) && (h > 640) ){
	option.cent_Y = 320 + (h - 640) / 2 -
	  (h - 640) * (my_nib[false] - 1)/(nibMax[false] - 1);
	
      }
      if ( (nibMax[true] != 1) && (w > 480) ){
	option.cent_X = 240 - (w - 480) /2 +
	  (w - 480) * (nibH - 1)/(nibMax[true] - 1);
      }
    }
    break;
  default:
    break;
  }


  if (imageRotation)
    makePaintImageR();
  else
    makePaintImageN();
  repaint();
}


// make paint image for view style(480x640)
// convert scaled image to paint image
// scaled image's width & height -> 480x640
// triming only
void BookReader::makePaintImageN()
{

  short *fb;

  if( !paintImage.create(480,640,16) ){
    fprintf(stderr,"Not enough memory");
    close();
    return;
  }

  fb = (short*)paintImage.bits();

  int src_width = scaledImage.width();
  int src_height = scaledImage.height();
  
  int src_x1 = option.cent_X - (src_width/2); 
  int src_x2 = option.cent_X + (src_width/2);
  int src_y1 = option.cent_Y - (src_height/2);
  int src_y2 = option.cent_Y + (src_height/2);

  int my_bgColor = (( ((option.bgcolor>>16) & 0xff) >> 3) << 11) |
                   (( ((option.bgcolor>> 8) & 0xff) >> 2) << 5)  |
                   (( ((option.bgcolor>> 0) & 0xff) >> 3) << 0);

  if(src_x1 < 0)
    src_x1 = 0;
  if(src_x2 >= 480)
    src_x2 = 479;
  if(src_y1 < 0)
    src_y1 = 0;
  if(src_y2 >= 640)
    src_y2 = 640;
  
  int x=0,y=0;

  //
  // Paint frame buffer from scaledImage data 
  //
  while (y <src_y1){
    for(x=0;x<480;x++)
      *fb++=my_bgColor;
    y++; 
  }
  for( y = src_y1 ; y < src_y2 ; y++){
    x=0;
    while(x<src_x1){
      *fb++=my_bgColor;
      x++;
    }
    short *tmp= (short*)scaledImage.scanLine(src_height/2 - option.cent_Y + y);    
    if ( (src_width/2 - option.cent_X) >0 )
      tmp += src_width/2 - option.cent_X;
    for( x = src_x1 ; x <=src_x2 ; x++){
      *fb++=*tmp++;
    }
    while(x<480){
      *fb++=my_bgColor;
      x++;
    }
  }
  while (y <640){
    for(x=0;x<480;x++)
      *fb++=my_bgColor;
    y++; 
  }

}


// make paint image for input style(640x480)
// convert scaled image to paint image
// scaled image's width & height -> 640x480
// rotation and triming 
void BookReader::makePaintImageR()
{

  short *fb;

  if( !paintImage.create(480,640,16) ){
    fprintf(stderr,"Not enough memory");
    close();
    return;
  }

  fb = (short*)paintImage.bits();

  int src_width = scaledImage.width();
  int src_height = scaledImage.height();

  int src_x1 = option.cent_X - (src_width/2); 
  int src_x2 = option.cent_X + (src_width/2);
  int src_y1 = option.cent_Y - (src_height/2);
  int src_y2 = option.cent_Y + (src_height/2);

  int my_bgColor = (( ((option.bgcolor>>16) & 0xff) >> 3) << 11) |
                   (( ((option.bgcolor>> 8) & 0xff) >> 2) << 5)  |
                   (( ((option.bgcolor>> 0) & 0xff) >> 3) << 0);

  if(src_x1 < 0)
    src_x1 = 0;
  if(src_x2 >= 640)
    src_x2 = 639;
  if(src_y1 < 0)
    src_y1 = 0;
  if(src_y2 >= 480)
    src_y2 = 480;

  int x=0,y=0;
  short *org_fb;
  org_fb=fb;

  fb+=479;

  //
  // Rotate and paint frame buffer from scaledImage data 
  //
  while (y <src_y1){
    for(x=0;x<640;x++){
      *fb=my_bgColor;
      fb+=480;
    }
    y++; 
    fb=org_fb+479-y;
  }
  
  for( y = src_y1 ; y < src_y2 ; y++){
    fb=org_fb+479-y;
    x=0;
    while(x<src_x1){
      *fb=my_bgColor;
      fb+=480;
      x++;
    }
    short *tmp= (short*)scaledImage.scanLine(src_height/2 - option.cent_Y + y);    
    if ((src_width/2 - option.cent_X) > 0)
      tmp += src_width/2 - option.cent_X;
    for( x = src_x1 ; x <= src_x2 ; x++){
      *fb=*tmp++;
      fb+=480;
    }
    while(x<640){
      *fb=my_bgColor;
      fb+=480;
      x++;
    }

  }

  fb=org_fb+479-y;
  while (y <480){
    for(x=0;x<640;x++){
      *fb=my_bgColor;
      fb+=480;
    }
    y++; 
    fb=org_fb+479-y;
  }
    
}



void BookReader::enableFullscreen() {
  // Make sure size is correct
  if(!my_has_fullscreen) {
    //    setFixedSize(qApp->desktop()->size());
    // This call is needed because showFullScreen won't work
    // correctly if the widget already considers itself to be fullscreen.
    showNormal();
    // This is needed because showNormal() forcefully changes the window
    // style to WSTyle_TopLevel.
    setWFlags(WStyle_Customize | WStyle_NoBorder);
    // Enable fullscreen.
    showFullScreen();
    my_has_fullscreen = true;
  }
}



int BookReader::applyAutoScale(const int scale, const int src_width, const int src_height ){

  //
  // auto scale setting
  //
  int my_scale = scale;

  if (option.autoscale != AS_None){
    int x_ratio, y_ratio;
    if(imageRotation){
      x_ratio = (640 * 100)/src_width;
      y_ratio = (480 * 100)/src_height;
    }else{
      x_ratio = (480 * 100)/src_width;
      y_ratio = (640 * 100)/src_height;
    } 
    switch (option.autoscale) {
    case AS_Width:
      my_scale =  x_ratio;
      break;
    case AS_Height:
      my_scale =  y_ratio;
      break;
    case AS_Both:
    default:
      my_scale = x_ratio < y_ratio ? x_ratio : y_ratio;
      break;
    }
  }
  
  if (my_scale >=2000){
    my_scale = 2000;
  }
  return my_scale;
} 



int BookReader::loadFile( const char *filename, const int percent )
{
  FILE * infile;		/* source file */
  static int my_scale;
  my_scale = percent;
  struct Image_Info iinfo; 
  char header[16];
  char *sourceImage = 0;
  short *dst = 0;
  QImage sourceQImage;

  for(int i = 0; i < 16; i++)
    header[i] = '\0';

  ImageType type = NULL_IMAGE;

  if ((infile = fopen(filename, "rb")) != NULL) {
    fread(header, 1, 16, infile);
    fclose(infile);
  }else{
    fprintf(stderr, "can't open %s\n", filename);
  }

  if (check_if_png(header)){
    type = PNG_IMAGE;
  }else if (check_if_jpeg(header)){
    type = JPEG_IMAGE;
  }else if (check_if_tiff(header)){
    type = TIFF_IMAGE;
    //  }else if (check_if_gif(header)){
    //    type = GIF_IMAGE;
  }else if (sourceQImage.load(filename)){
    type = USE_QIMAGE;
    iinfo.width = sourceQImage.width();
    iinfo.height = sourceQImage.height();
  }else {
    fprintf(stderr, "unrecognized file %s\n", filename);
  }

  
  //
  // get image infomation ( width & height )
  //

  if (type == JPEG_IMAGE){
    if (loadJpegInit(filename, &(iinfo.width), &(iinfo.height) ) == false){
      type = NULL_IMAGE;
    }
  }else if(type == PNG_IMAGE){
    if (loadPNGInit(filename,  &(iinfo.width), &(iinfo.height) ) == false){
      type = NULL_IMAGE;
    }
  }else if(type == TIFF_IMAGE){
    if (loadTIFFInit(filename,  &(iinfo.width), &(iinfo.height) ) == false){
      type = NULL_IMAGE;
    }
    //  }else if(type == GIF_IMAGE){
    //    if (loadGIFInit(filename,  &(iinfo.width), &(iinfo.height) ) == false){
    //      type = NULL_IMAGE;
    //    }
  }

  if (type != NULL_IMAGE){
    iinfo.src_width = ( iinfo.width * 
			( option.nib.highH - option.nib.lowH) ) / 100;
    iinfo.src_height = ( iinfo.height * 
			 (  option.nib.highV - option.nib.lowV) ) / 100;
    iinfo.start_h = (iinfo.width * option.nib.lowH) /100 ;
    iinfo.end_h = iinfo.start_h + iinfo.src_width ;
    iinfo.start_v = (iinfo.height * option.nib.lowV) /100 ;
    iinfo.end_v = iinfo.start_v + iinfo.src_height ;
  
    my_scale = applyAutoScale(my_scale, iinfo.src_width, iinfo.src_height );

    iinfo.dst_width = (iinfo.src_width * my_scale) / 100;
    iinfo.dst_height = (iinfo.src_height * my_scale) / 100;
  }

  if((type == JPEG_IMAGE) && (my_scale <= 100))
    type = JPEG_IMAGE_S;

  //
  // make source image buffer and scaled image buffer
  //

  set_new_handler(0);

  if (type != NULL_IMAGE){
    if(type != JPEG_IMAGE_S){
      sourceImage = new char[iinfo.width * iinfo.height * 3];
      if( sourceImage == 0 ){
	fprintf(stderr,"Not enough memory source image\n");
	type = OVER_SIZE;
      }
      iinfo.image = sourceImage;
    }

    if( !scaledImage.create(iinfo.dst_width,iinfo.dst_height,16) ){
      fprintf(stderr,"Not enough memory dest image\n");
      type = OVER_SIZE;
    }
    dst = (short*)scaledImage.bits();
  }


  //  create error message to image


  if( type == NULL_IMAGE || type == OVER_SIZE){
    if( !scaledImage.create(240, 60, 16) ){
      fprintf(stderr,"Not enough memory dest image\n");
    }else {
      uchar *srcBits;
      short *dstBits;
      int my_bgColor = (( ((option.bgcolor>>16) & 0xff) >> 3) << 11) |
	(( ((option.bgcolor>> 8) & 0xff) >> 2) << 5)  |
	(( ((option.bgcolor>> 0) & 0xff) >> 3) << 0);

      QPixmap pm(240, 60, 1);
      QImage im;
      QPainter p(&pm);

      p.setPen(QColor(255,255,255));
      p.setFont(QFont("helvetica",16));
      if(type != OVER_SIZE){
	p.drawText(2,20,"Not Enough Memory.");
	p.drawText(6,40,"File is too large.");
      }else{
	p.drawText(10,20,"Can not read file or");
	p.drawText(2,40,"unrecognized file format.");
      }

      im = pm.convertToImage();
      srcBits = im.bits();
      dstBits = (short*)scaledImage.bits();
      for(int i = 0; i<60 ;i++){
	for(int j = 0;j<240/8;j++){
	  for(int k=0; k<8; k++){
	    *dstBits++ = ((*srcBits >> k) & 0x01) ? 0x0 : my_bgColor;
	  }
	  srcBits++;
	}
      }
    }

    return my_scale;
  }

  if( type == JPEG_IMAGE_S) {
    makeJpegS( dst, my_scale, &iinfo);
    return my_scale;
  }else if(type == JPEG_IMAGE) {
    makeJpeg( &iinfo );
  }else if(type == PNG_IMAGE) {
    makePNG( &iinfo );
  }else if(type == TIFF_IMAGE) {
    makeTIFF( &iinfo );
    //  }else if(type == GIF_IMAGE) {
    //    makeGIF( &iinfo );
  }else if(type == USE_QIMAGE) {
    char *srcBits;
    QImage tmpQImage = sourceQImage.convertDepth(32);
    srcBits = (char*)tmpQImage.bits();
    for (int i = 0; i < iinfo.height; i++){
      for(int j = 0; j < iinfo.width; j++){
	*(iinfo.image)++ = *srcBits++; 
	*(iinfo.image)++ = *srcBits++;
	*(iinfo.image)++ = *srcBits++; 
	srcBits++;
      }
    }
  }

  iinfo.image = sourceImage;

  if(my_scale <= 100)
    smalledImage(&iinfo, dst, my_scale);
  else
    largedImage(&iinfo, dst, my_scale);

  if(sourceImage != NULL){
    delete sourceImage;
    sourceImage = 0;
  }

  return my_scale;
}



void smalledImage(Image_Info *iinfo, short *dst, int my_scale)
{
  //
  //  Convert sourceImage to scaledImage
  //

  int i,j;
  char *src;
  int *line = new int[iinfo->width * 3];
  int *lineData;
  int lines = 0;
  int prev_line = -1;
  int total_line = 0;

  lineData = line;
  for(i = 0; i < iinfo->width ; i++){
    *lineData++ = 0;
    *lineData++ = 0;
    *lineData++ = 0;
  }

  for(i = iinfo->start_v; i < iinfo->end_v; i++) {
    src = iinfo->image + i * iinfo->width * 3;

    lineData = line;
    for(j = 0; j < iinfo->width ; j++){
      *lineData++ += *src++;
      *lineData++ += *src++;
      *lineData++ += *src++;
    }
    lines++;

   if ( prev_line == ( i * my_scale)/100 )
      continue;
  
    prev_line = ( i * my_scale)/100;

    total_line++;
    if (total_line > iinfo->dst_height)
      break;
   
    lineData = line;

    int r = 0, g = 0, b = 0;
    int pixels = 0, prev_pixel = -1;
    short *last_pixel;
    last_pixel = dst + iinfo->dst_width; 


    lineData += iinfo->start_h * 3;
    for(j = iinfo->start_h; j < iinfo->end_h; j++) {
      pixels++;
      r += *lineData++;
      g += *lineData++;
      b += *lineData++;
      if( prev_pixel == ((j * my_scale) / 100) )
	continue;
      
      prev_pixel = ((j * my_scale) / 100);
      
      *dst++ = ((( (r/(lines*pixels)) >> 3) << 11)|
		(( (g/(lines*pixels)) >> 2) << 5) |
		(( (b/(lines*pixels)) >> 3) << 0))  ;
      pixels = 0;
      r=0;g=0;b=0;
      if (dst >= last_pixel)
	break;
      
    }

    lineData = line;
    for(j = 0; j < iinfo->width ; j++){
      *lineData++ = 0;
      *lineData++ = 0;
      *lineData++ = 0;
    }
    lines = 0;

  }


  if(line != NULL){
    delete line;
    line = 0;
  }

}

void largedImage(Image_Info *iinfo, short *dst, int my_scale)
{

  int i, j;
  int r, g, b;
  char *src;
  for ( i = 0; i < iinfo->dst_height; i++){
    for( j = 0; j < iinfo->dst_width; j++){
      src = ((i + iinfo->start_v) * 100 / my_scale) * iinfo->width * 3 + 
	( ( (iinfo->start_h + j ) * 100) / my_scale)  * 3 + iinfo->image;
      r = *src++;
      g = *src++;
      b = *src++;

      *dst++ = ((( r >> 3) << 11)|
		(( g >> 2) << 5) |
		(( b >> 3) << 0) ) ;

    }
  }

}
