/*
 $Revision: 1.29.2.8 $
 $Date: 2005/01/19 07:08:52 $
*/
#ifndef BOOKREADER_H
#define BOOKREADER_H
#include "brcontrol.h"

#include <qwidget.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qimage.h>
#include <qdirectpainter_qws.h>
#include <qvaluelist.h>

typedef unsigned short  Uint16;

class BookReader : public QWidget
{ 
  void QueueKey(QKeyEvent *e, int pressed);

  Q_OBJECT

    
    public:
       BookReader( const BR_Option );

      ~BookReader();
      int loadFile( const char *filename, const int percent );
      int loadFileSet( int );
      int loadPNGFile( const char *filename, const int percent );
      int applyAutoScale( const int, const int, const int);

      void setWFlags(WFlags flags) {
	QWidget::setWFlags(flags);
      }

      void brControlCreate();

      void nextImage();
      void prevImage();
      void rotationImage();

      bool showScreen;
      void changeStyle();

    public slots:
      void testView();
      void makeBookmark();
    signals:
      void closed();

    protected:
      void paintEvent( QPaintEvent * );
      void keyPressEvent( QKeyEvent *e )  { QueueKey(e, 1); }
      void keyReleaseEvent( QKeyEvent *e) { QueueKey(e, 0); } 
      void closeEvent( QCloseEvent * );
      void mousePressEvent( QMouseEvent *);
      void mouseReleaseEvent( QMouseEvent *);
      void mouseDoubleClickEvent( QMouseEvent *);
      void focusInEvent(QFocusEvent *);

      bool event(QEvent*);

    private:
      void makePaintImage();
      void makePaintImageN();
      void makePaintImageR();
      void enableFullscreen();

      BR_Option option;

      QString loadedFilename;
      QString currentDir;
      QStringList::ConstIterator fileListIt;

      QImage scaledImage;
      QImage paintImage;

      QDirectPainter *my_painter;
      
      BRControl *brControl;

      int org_ratio;
      ASFlags org_autoscale;

      bool imageRotation;

      int prev_x;
      int prev_y;

      int org_centX;
      int org_centY;

      // nibble mode 
      // true  -> Horizonal nibble
      // false -> Vertical nibble  

      int my_nib[2];    // current nibble mode posision
      int nibMax[2];    // nibble mode divider  
      bool horizPrimary;

      WFlags my_has_fullscreen;


 };

extern bool loadPNGInit( const char *infile, int *width, int *height);
extern void makePNG( struct Image_Info *iinfo);
extern int check_if_png( char *header);

extern bool loadJpegInit( const char *infile, int *width, int *height);
extern void makeJpeg( struct Image_Info *iinfo);
extern void makeJpegS( short *dst, const int my_scale, struct Image_Info *iinfo);
extern int check_if_jpeg( char *header);

//extern bool loadGIFInit( const char *infile, int *width, int *height);
//extern void makeGIF( struct Image_Info *iinfo);
//extern int check_if_gif( char *header);

extern bool loadTIFFInit( const char *infile, int *width, int *height);
extern void makeTIFF( struct Image_Info *iinfo);
extern int check_if_tiff( char *header);

void smalledImage(Image_Info *iinfo, short *dst, int my_scale);
void largedImage(Image_Info *iinfo, short *dst, int my_scale);


 
#endif // BOOKREADER_H

