#include "directory.h"
#include "setbg.h"
#include "setscale.h"
#include "setcenter.h"
#include "setnibble.h"
#include "setextension.h"

#include <qdir.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qapplication.h>
#include <qpe/qpeapplication.h>
#include <qtextstream.h>
#include <qpe/global.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* 
 *  Constructs a Example which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */

Directory::Directory( const QString & dir_name, QWidget* parent,  const char* name, WFlags fl)
    : QWidget( parent, name, fl )
{

  //
  // standard setting
  //
  br = 0;
  option.bgcolor=0xffffff;    // Initial background color -> white
  option.autoscale = AS_Both;         // Initial scale -> auto scale
  option.cent_X = 320;
  option.cent_Y = 240;
  option.nib.mode = NIB_None;
  option.nib.divH = 1;
  option.nib.divV = 1;
  option.nib.left = false;
  option.nib.novel = false;
  option.nib.lowH = 0;
  option.nib.highH = 100;
  option.nib.lowV = 0;
  option.nib.highV = 100;

  option.validExt = "jpg png gif tiff bmp xbm xpm";


  dirName = new QLineEdit( this, "DirName" );
  dirName->setGeometry( QRect( 0, 30, width(), 27 ) ); 
  dirName->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
				       (QSizePolicy::SizeType)7,
				       dirName->sizePolicy().hasHeightForWidth() ) );
  dirName->setText(dir_name);
  connect(dirName, SIGNAL(textChanged(const QString &)),
	  this, SLOT(showDirectory()) );
  

  showDir = new QListView( this, "ShowDir" );
  showDir->setGeometry( QRect( 0, 60, width(), height()-60 ) ); 
  showDir->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)3,
				       showDir->sizePolicy().hasHeightForWidth() ) );
  showDir->addColumn("FileName");
  showDir->setColumnWidthMode(0, QListView::Manual);
  showDir->setColumnWidth(0, 200);
  showDir->addColumn("Ext");
  showDir->addColumn("Size");
  showDir->setColumnAlignment(2, Qt::AlignRight);
  showDir->addColumn("Perm");
  showDir->addColumn("Date");
  
  showDir->setFocus();
  
  connect(showDir, SIGNAL(doubleClicked(QListViewItem*)),
	  this, SLOT(fileView(QListViewItem*)) );
  connect(showDir, SIGNAL(returnPressed(QListViewItem*)),
	  this, SLOT(fileView(QListViewItem*)) );
  

  menuBar = new DirMenuBar( this, "MenuBar");
  connect(menuBar, SIGNAL(menuChecked(MenuItems)),
	  this, SLOT(menuChecked(MenuItems)) );

  //
  // loading preconfig
  //
  configSetting(Starting);

  showDirectory();

}

/*  
 *  Destroys the object and frees any allocated resources
 */

Directory::~Directory()
{
  // no need to delete child widgets, Qt does it all for us
}

bool Directory::event(QEvent *e)
{
  static int prev_event = 0;
  static int prev_width = 0;

  if(br != 0){
    if(prev_width != width() ) {
      br->changeStyle();
    }else if(e->type() == 12 && prev_event == 28){
      br->hide();
      br->showScreen = true;
      br->show();
    }
  }
  
  prev_event = e->type();
  prev_width = width();

  return QWidget::event(e);
}



/*
 * MenuBar check event
 */

void Directory::menuChecked(MenuItems checkedMenu)
{
  switch (checkedMenu){
  case FileOpen:
    fileView(showDir->selectedItem());
    break;
  case ProgQuit:
    close();
    break;
  case SetBG:
    {
      SetBGDialog * setbg = new SetBGDialog(option);
      setbg->show();
      if( setbg->exec() ){
	option.bgcolor = setbg->rgbColor();
      }
      delete setbg;
    }
    break;
  case SetExtension:
    {
      SetEXTDialog * setext = new SetEXTDialog(option);
      setext->show();
      if( setext->exec() ){
	option.validExt = setext->getExt();
      }
      delete setext;
    }
    break;
    
  case SetScale:
    {
      SetScaleDialog *setscale = new SetScaleDialog(option);
      setscale->show();
      if ( setscale->exec() ){
	option.ratio = setscale->getScale();
      option.autoscale = setscale->getASFlag();
      }
      delete setscale;
    }
    break;
  case SetCenter:
    {
      SetCenterDialog *setcenter = new SetCenterDialog(option);
      setcenter->show();
      if ( setcenter->exec() ){
	option.cent_X = setcenter->x();
	option.cent_Y = setcenter->y();
      }
      delete setcenter;
    }
    break;
  case SetNibble:
    {
      SetNibbleDialog *setnibble = new SetNibbleDialog(option);
      setnibble->show();
      if ( setnibble->exec() ){
	option.nib = setnibble->getNIB_Opt();
      }
      delete setnibble;
    }
    break;
  case LoadSetting:
    configSetting(Standard);
    break;
  case SaveSetting:
    saveSetting(Standard);
    break;
  case SaveSettingLocal:
    saveSetting(Local);
    break;
  case OutFocus:
    showDir->setFocus();
    break;
  default:
    break;
  }

}

void Directory::showDirectory()         // Make Directory List & show 
{
  QString dir_name = dirName->text();   // Current Directory
   
   QDir cur_dir( dir_name );
   cur_dir.setFilter(QDir::All | QDir::Hidden );
   
   if(cur_dir.exists() != TRUE){
     return;
   }
   showDir->clear();
   cur_dir_num=cur_dir.count();
   
   for ( int i=0; i<cur_dir_num; i++ ){       //Reading File infomation
     QFileInfo f(cur_dir,cur_dir[i]);
     QListViewItem *file;
     
     QString fname;
     QString ext;
     QString size;
     QString perm;
     QString fdate;
     QString isDir;
     
     if (cur_dir[i].left(1) != "." && f.extension(FALSE) != "") {
       ext = "."+f.extension(FALSE);
       fname = cur_dir[i].replace(cur_dir[i].length()-ext.length(), ext.length(), "");
     }else {
       ext = "";
       fname = cur_dir[i];
     }
     perm  = (f.isDir() ? "d" : "-");         //permission check
     perm += (f.isReadable() ? "r" : "-");
     perm += (f.isWritable() ? "w" : "-");
     perm += (f.isExecutable() ? "x" : "-");
     perm += (f.isSymLink() ? "s" : "-");
     
     if (f.isDir()){                          //directory check
       size = "<DIR>";
       isDir = "D";
     }else {
       size = QString::number(f.size());
       isDir = "N";
     }
     
     file = new QListViewItem(showDir, fname, ext, size, perm, f.lastModified().toString(), isDir+cur_dir[i]);
     showDir->insertItem(file);
   }
   QString caption;
   showDir->setSorting(5);
   showDir->sort();
   caption = QString::QString("Files ") + " ("+QString::number(cur_dir_num)+")";
   setCaption( caption );

   configSetting(Local);

}

/*
  Directory Change
*/

void Directory::changeDir(const QString & chgDir)
{
  if (chgDir == "."){
    return;
  }
  if (chgDir == ".."){ 
    QDir d(dirName->text());
    if(d.cdUp() == TRUE){
      dirName->setText(d.absPath());
    }      
    return;
  }
  if (chgDir.left(1) == "/" ){
    QDir d(chgDir);
    if(d.exists() == TRUE){
      dirName->setText(d.absPath());
    }      
    return;
  }
    
  QDir d(dirName->text()+"/"+chgDir);
  if(d.exists() == TRUE){
    dirName->setText(d.absPath());
  }      
  
}


/*
  Directory Class Events
*/

void Directory::resizeEvent(QResizeEvent * )
{
  showDir->setGeometry( QRect( 0, 60, width(), height()-60));
  dirName->setGeometry( QRect( 0, 30, width(), 27 ));
  
}

void Directory::keyPressEvent(QKeyEvent *e)
{
  if(e->key() == Key_F11)
    menuBar->setFocus();
}


void Directory::fileView(QListViewItem * item)
{
  if( item != 0 ){  
    if( (item->text(3).left(1) == "d") ){
      changeDir(item->text(0));
      return;
    }
  }else if ( !menuBar->set_bmCheck() ){
    return;
  }

    
  QString buf;
  option.rot = false;
  
  QFile f( "/tmp/qtembedded-zaurus/QtEmbedded-0.spec");  // Rotation check
  if ( f.open(IO_ReadOnly) ) {
    if( f.readLine(buf, 80) != -1)
      if( strstr( buf.latin1(), "270") != NULL){
	option.rot = true;
      }
    f.close();
  }
  
  fileList.clear();
  
  QListViewItemIterator it( showDir );             //Make file list
  QStringList listExt = QStringList::split(" ", option.validExt);

  for ( ; it.current() ; it++){
    for ( QStringList::Iterator sit = listExt.begin(); sit != listExt.end(); sit++ ){
      if(it.current()->text(1).upper() == ("."+(*sit).upper()) ){
	fileList.append(it.current()->text(0)+it.current()->text(1));
      }
    }
  }
  option.filelist = &fileList;

  if(br != 0){
    delete br;
  }

  option.filename = "";

  QString filename;
  if( menuBar->set_bmCheck() ){
    QFile f(dirName->text()+"/.bookmark");
    if (f.open(IO_ReadOnly)){
      QTextStream ts(&f);
      option.filename = ts.read();
      QFileInfo f(dirName->text()+"/"+filename);
      if ( !f.exists() ){
	for ( QStringList::Iterator sit = listExt.begin(); sit != listExt.end(); sit++ ){
	  if( (item->text(1).upper() == ("." + (*sit).upper()) ) )
	    option.filename = item->text(0)+item->text(1);
	}
      }
    }else{
      for ( QStringList::Iterator sit = listExt.begin(); sit != listExt.end(); sit++ ){
	if( (item->text(1).upper() == ("." + (*sit).upper()) ) )
	  option.filename = item->text(0)+item->text(1);
      }
    }
    
  }else{
    for ( QStringList::Iterator sit = listExt.begin(); sit != listExt.end(); sit++ ){
      if( (item->text(1).upper() == ("." + (*sit).upper()) ) )
	option.filename = item->text(0)+item->text(1);
    }
  }
  
  if( option.filename == "" )
    return;

  option.dir_name = dirName->text();
  option.auto_bookmark = menuBar->set_auto_bmCheck();

  br=new BookReader(option);
  br->show();
  connect(br, SIGNAL(closed()), this, SLOT(fileViewClosed() ));

}

void Directory::fileViewClosed()
{
  delete br;
  br=0;
}

void Directory::configSetting(ConfType type )
{
  QString filName;
  QString message;

  if(type == Local){
    filName = dirName->text() + "/.brconf";
  } else {
    filName = "/home/zaurus/Settings/bookreader.conf";
  }
  
  QFile f(filName);
  if( !f.exists())
    return;

  if ( f.open(IO_ReadOnly) ){
    QString s;
    QTextStream ts(&f);

    while ( (s = ts.readLine() ) != NULL ){
      
      QStringList list = QStringList::split(',', s, FALSE);

      if (type == Starting){
	if (list[0] == "HomeDir"){
	  dirName->setText(list[1].stripWhiteSpace());
	}
      }

      if (list[0] == "Extension"){
	option.validExt = list[1].stripWhiteSpace();
      }

      if (list[0] == "Scale"){
	bool ok;
	int tmp_scale;
	tmp_scale = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.ratio = tmp_scale;
      }

      if (list[0] == "AutoScale"){
	bool ok;
	int tmp_scale;
	tmp_scale = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.autoscale = (ASFlags)tmp_scale;
      }

      if (list[0] == "BGColor"){
	bool ok;
	int tmp_color;
	tmp_color = list[1].toInt( &ok, 16);
	if( ok ) 
	  option.bgcolor = tmp_color;
      }

      if (list[0] == "CenterX"){
	bool ok;
	int tmp_x;
	tmp_x = list[1].toInt( &ok, 10);
	if( ok ) 
	 option.cent_X = tmp_x;
      }

      if (list[0] == "CenterY"){
	bool ok;
	int tmp_y;
	tmp_y = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.cent_Y = tmp_y;
      }

      if (list[0] == "AutoBookmark"){
	bool ok;
	int tmp_ab;
	tmp_ab = list[1].toInt( &ok, 10);
	if( ok && tmp_ab) 
	  menuBar->set_auto_bmCheck(true);
	else
	  menuBar->set_auto_bmCheck(false);
      }

      if (list[0] == "Bookmark"){
	bool ok;
	int tmp_ab;
	tmp_ab = list[1].toInt( &ok, 10);
	if( ok && tmp_ab) 
	  menuBar->set_bmCheck(true);
	else
	  menuBar->set_bmCheck(false);
      }

      if (list[0] == "NibbleMode"){
	bool ok;
	int tmp_mode;
	tmp_mode = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.mode = (NIBFlags)tmp_mode;
      }

      if (list[0] == "NibbleH"){
	bool ok;
	int tmp_H;
	tmp_H = list[1].toInt( &ok, 10);
	if( ok ) 
	 option.nib.divH = tmp_H;
      }

      if (list[0] == "NibbleV"){
	bool ok;
	int tmp_V;
	tmp_V = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.divV = tmp_V;
      }

      if (list[0] == "NibLeft"){
	bool ok;
	int tmp_Left;
	tmp_Left = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.left = tmp_Left;
      }

      if (list[0] == "NibNovel"){
	bool ok;
	int tmp_Novel;
	tmp_Novel = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.novel = tmp_Novel;
      }

      if (list[0] == "NibbleLowH"){
	bool ok;
	int tmp_lowH;
	tmp_lowH = list[1].toInt( &ok, 10);
	if( ok ) 
	 option.nib.lowH = tmp_lowH;
      }

      if (list[0] == "NibbleHighH"){
	bool ok;
	int tmp_highH;
	tmp_highH = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.highH = tmp_highH;
      }

      if (list[0] == "NibbleLowV"){
	bool ok;
	int tmp_lowV;
	tmp_lowV = list[1].toInt( &ok, 10);
	if( ok ) 
	 option.nib.lowV = tmp_lowV;
      }

      if (list[0] == "NibbleHighV"){
	bool ok;
	int tmp_highV;
	tmp_highV = list[1].toInt( &ok, 10);
	if( ok ) 
	  option.nib.highV = tmp_highV;
      }

    }

    if(type == Local){
      message = "Loaded Local Setting";
      Global::statusMessage(message);
    }else if(type == Standard){
      message = "Loaded Standard Setting";
      Global::statusMessage(message);
    }

  f.close();
  }

}

void Directory::saveSetting(ConfType type )
{

  QString filName;
  QString message;
  
  if(type == Local){
    filName = dirName->text() + "/.brconf";
  } else {
    filName = "/home/zaurus/Settings/bookreader.conf";
  }
  
  QFile f(filName);

  if ( f.open(IO_WriteOnly) ){
    QString s;

    QTextStream ts(&f);

    s = "HomeDir,"+ dirName->text()+"\n";
    ts << s;

    s = "Extension,"+ option.validExt+"\n";
    ts << s;

    s = "Scale,"+ QString::number(option.ratio, 10)+"\n";
    ts << s;

    s = "AutoScale,"+ QString::number(option.autoscale, 10)+"\n";
    ts << s;

    s = "BGColor,"+ QString::number(option.bgcolor, 16)+"\n";
    ts << s;

    s = "CenterX,"+ QString::number(option.cent_X, 10)+"\n";
    ts << s;

    s = "CenterY,"+ QString::number(option.cent_Y, 10)+"\n";
    ts << s;

    s = "AutoBookmark,"+ QString::number(menuBar->set_auto_bmCheck(), 10)+"\n";
    ts << s;

    s = "Bookmark,"+ QString::number(menuBar->set_bmCheck(), 10)+"\n";
    ts << s;

    s = "NibbleMode,"+ QString::number(option.nib.mode, 10)+"\n";
    ts << s;

    s = "NibbleH,"+ QString::number(option.nib.divH, 10)+"\n";
    ts << s;

    s = "NibbleV,"+ QString::number(option.nib.divV, 10)+"\n";
    ts << s;

    s = "NibLeft,"+ QString::number(option.nib.left, 10)+"\n";
    ts << s;

    s = "NibNovel,"+ QString::number(option.nib.novel, 10)+"\n";
    ts << s;

    s = "NibbleLowH,"+ QString::number(option.nib.lowH, 10)+"\n";
    ts << s;

    s = "NibbleHighH,"+ QString::number(option.nib.highH, 10)+"\n";
    ts << s;

    s = "NibbleLowV,"+ QString::number(option.nib.lowV, 10)+"\n";
    ts << s;

    s = "NibbleHighV,"+ QString::number(option.nib.highV, 10)+"\n";
    ts << s;

    if(type == Local)
      message = "Saved Local Setting";
    else
      message = "Saved Standard Setting";

    Global::statusMessage(message);

  f.close();
  }else {
    message = "Can't Save Setting";
    Global::statusMessage(message);
  }

}

