/***************************************************
      load PNG routine

    Loading and scaling to scaledImage 16bit QImage

***************************************************/   
#include <sys/types.h> 
#include <unistd.h> 
#include <stdlib.h>
#include "loadpng.h"


#ifndef png_jmpbuf
#  define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
#endif

#define PNG_BYTES_TO_CHECK 4

int check_if_png(char *header)
{
  return(!png_sig_cmp((png_bytep)header, (png_size_t)0, PNG_BYTES_TO_CHECK));
}

bool loadPNGInit( struct PNG_handler *hdr, FILE *infile,
		  int *image_width, int *image_height)
{
  unsigned int sig_read = 0;
  int bit_depth, color_type, interlace_type;
  
  hdr->png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  
  if (hdr->png_ptr == NULL){
    fclose(infile);
    return false;
  }
  
  hdr->info_ptr = png_create_info_struct(hdr->png_ptr);
  if (hdr->info_ptr == NULL){
      fclose(infile);
      png_destroy_read_struct(&(hdr->png_ptr), png_infopp_NULL, png_infopp_NULL);
      return false;
  }
  
  if (setjmp(png_jmpbuf(hdr->png_ptr))){
    png_destroy_read_struct(&(hdr->png_ptr), &(hdr->info_ptr), png_infopp_NULL);
    fclose(infile);
    return false;
  }
  
  png_init_io(hdr->png_ptr, infile);
  png_set_sig_bytes(hdr->png_ptr, sig_read);
  
  png_read_info(hdr->png_ptr, hdr->info_ptr);
  png_get_IHDR(hdr->png_ptr, hdr->info_ptr, (png_uint_32*)image_width, (png_uint_32*)image_height,
	       &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);
  hdr->number_passes = png_set_interlace_handling(hdr->png_ptr);
  png_set_strip_16(hdr->png_ptr);
  if (color_type == PNG_COLOR_TYPE_PALETTE)
    png_set_expand(hdr->png_ptr);
  png_set_strip_alpha(hdr->png_ptr);
  png_read_update_info(hdr->png_ptr, hdr->info_ptr);

  hdr->buffer = (png_bytep) png_malloc(hdr->png_ptr, png_get_rowbytes(hdr->png_ptr, hdr->info_ptr));

  return true;
}

void makePNG( struct PNG_handler *hdr, const int my_scale, struct Image_Info *iinfo)
{
  
  uint *line = new uint[iinfo->image_width * 3];    
  uint *lineData;
  unsigned char *src=0;
  int prev_line = 0;
  int lines = 0;

  int current_line;
  int last_line;
  int pass;

  last_line = iinfo->image_height;

  lineData = line;
  for(int i = 0; i < iinfo->image_width ; i++){
    *lineData++ = 0;
    *lineData++ = 0;
    *lineData++ = 0;
  }

  for(pass = 0; pass < hdr->number_passes; pass++){
    for (current_line = 0; current_line < last_line; current_line++) {
      png_read_row(hdr->png_ptr, hdr->buffer, png_bytep_NULL);
      
      if(current_line < iinfo->start_v)
	continue;
      
      //
      // skip line
      //
      lineData = line;

      src = (unsigned char*)(hdr->buffer);

      for(int i = 0; i < iinfo->image_width ; i++){
	*lineData++ += *src++;
	*lineData++ += *src++;
	*lineData++ += *src++;
      }
      lines++;
      
      if ( prev_line == ( (current_line-1) * my_scale)/100 )
	continue;
      
      //
      // draw line number
      //
      prev_line = ( (current_line-1) * my_scale)/100;
      if(current_line > (iinfo->src_height + iinfo->start_v - 1 ) )
	continue;
      
      lineData = line;
      
      int r = 0, g = 0, b = 0;
      int i;
      int pixels = 0, prev_pixel = -1;
      short *last_pixel;
      last_pixel = (iinfo->dst) + iinfo->dst_width; 
      
      lineData += iinfo->start_h * 3;
      for( i = iinfo->start_h; i < iinfo->end_h; i++  ) {
	pixels++;
	r += *lineData++;
	g += *lineData++;
	b += *lineData++;
	if( prev_pixel == ((i * my_scale) / 100) )
	  continue;
	
	prev_pixel = ((i * my_scale) / 100);
	
	*(iinfo->dst)++ = (( (r/(lines*pixels)) >> 3) << 11)|
	  (( (g/(lines*pixels)) >> 2) << 5) |
	  (( (b/(lines*pixels)) >> 3) << 0)  ;
	
	pixels = 0;
	r=0;g=0;b=0;
	if (iinfo->dst >= last_pixel)
	  break;
      }
      
      lineData = line;
      for(int i = 0; i < iinfo->image_width ; i++){
	*lineData++ = 0;
	*lineData++ = 0;
	*lineData++ = 0;
      }
      lines = 0;
    }
    delete line;
  }
  png_read_end(hdr->png_ptr, hdr->info_ptr);

  free(hdr->buffer);


  png_destroy_read_struct(&(hdr->png_ptr), &(hdr->info_ptr), png_infopp_NULL);


}
