/***************************************************
      load GIF routine

    Loading and scaling to scaledImage 16bit QImage

***************************************************/   
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "loadgif.h"

static int InterlacedOffset[] = { 0, 4, 2, 1 }, /* The way Interlaced image should. */
           InterlacedJumps[] = { 8, 8, 4, 2 };    /* be read - offsets and jumps... */

static void DumpScreen2RGB( unsigned char *imageScreen, Image_Info *iinfo, ColorMapObject *ColorMap);

static GifRecordType RecordType;
static GifByteType *Extension;
static GifPixelType *ScreenBuffer;
static GifFileType *GifFile;

int check_if_gif(char *header)
{
  char gif_header[5] = "GIF8";

  return !strncmp( header, gif_header, 4);
}

bool loadGIFInit( const char *filename, int *width, int *height)
{
  GifFile = DGifOpenFileName(filename);
  
  if (GifFile == NULL){
    return false;
  }
  
  *width = GifFile->SWidth;
  *height = GifFile->SHeight;


  return true;
}

void makeGIF( Image_Info *iinfo)
{

  int i, j, Row, Col, Width, Height, Count, ExtCode;
  GifByteType *Extension;

  ScreenBuffer = new unsigned char[(iinfo->width) * (iinfo->height)];
  if (ScreenBuffer == NULL){
    return;
  }
  memset(ScreenBuffer, ((iinfo->width) * (iinfo->height)), GifFile->SBackGroundColor);

  /* Scan the content of the GIF file and load the image(s) in: */
  do {
    if (DGifGetRecordType(GifFile, &RecordType) == GIF_ERROR) {
      fprintf(stderr, "unrecognized GIF file\n");
      return ;
    }

    switch (RecordType) {
    case IMAGE_DESC_RECORD_TYPE:
      if (DGifGetImageDesc(GifFile) == GIF_ERROR) {
	return;
      }

      Row = GifFile->Image.Top; /* Image Position relative to Screen. */
      Col = GifFile->Image.Left;
      Width = GifFile->Image.Width;
      Height = GifFile->Image.Height;

      if ( (Col + Width) > iinfo->width || (Row + Height) > iinfo->height) {
	fprintf(stderr, "Image is not confined to screen dimension, aborted.\n");
	return;
      }

      if (GifFile->Image.Interlace) {
	/* Need to perform 4 passes on the images: */
	for (Count = i = 0; i < 4; i++)
	  for (j = Row + InterlacedOffset[i]; j < Row + Height;
	       j += InterlacedJumps[i]) {
	    GifPixelType *screenLine = (GifPixelType*)ScreenBuffer + (Width * j) + Col;
	    if (DGifGetLine(GifFile, screenLine, Width) == GIF_ERROR) {
	      exit(EXIT_FAILURE);
	    }
	  }
      }
      else {
	for (i = 0; i < Height; i++) {
	  if (DGifGetLine(GifFile, (GifPixelType*)ScreenBuffer + (Width * Row++) + Col, Width) == GIF_ERROR) {
	    exit(EXIT_FAILURE);
	  }
	}
      }
      break;
    case EXTENSION_RECORD_TYPE:
      /* Skip any extension blocks in file: */
      if (DGifGetExtension(GifFile, &ExtCode, &Extension) == GIF_ERROR) {
	exit(EXIT_FAILURE);
      }
      while (Extension != NULL) {
	if (DGifGetExtensionNext(GifFile, &Extension) == GIF_ERROR) {
	  exit(EXIT_FAILURE);
	}
      }
      break;
    case TERMINATE_RECORD_TYPE:
      break;
    default:		    /* Should be traps by DGifGetRecordType. */
      break;
    }
  }
  while (RecordType != TERMINATE_RECORD_TYPE);
  
  /* Lets dump it - set the global variables required and do it: */

  ColorMapObject *ColorMap = (GifFile->Image.ColorMap
  	      ? GifFile->Image.ColorMap
  	      : GifFile->SColorMap);
  
  DumpScreen2RGB( ScreenBuffer, iinfo, ColorMap);

  delete ScreenBuffer;
  ScreenBuffer = 0;
  
  if (DGifCloseFile(GifFile) == GIF_ERROR) {
    exit(EXIT_FAILURE);
  }


}

static void DumpScreen2RGB( unsigned char *ScreenBuffer, Image_Info *iinfo, ColorMapObject *ColorMap)
{
  int i, j;
  
  for ( i = 0; i < iinfo->height; i++) {
    for ( j = 0 ; j < iinfo->width; j++){
      *(iinfo->image)++ = ColorMap->Colors[*ScreenBuffer].Red;
      *(iinfo->image)++ = ColorMap->Colors[*ScreenBuffer].Green;
      *(iinfo->image)++ = ColorMap->Colors[*ScreenBuffer++].Blue;
    }
  }

}
