
#include <sys/types.h> 
#include <unistd.h> 
#include "loadjpeg.h"

METHODDEF(void)
my_error_exit (j_common_ptr cinfo)
{
  my_error_ptr myerr = (my_error_ptr) cinfo->err;
  (*cinfo->err->output_message) (cinfo);
  longjmp(myerr->setjmp_buffer, 1);
}

int check_if_jpeg(char *header)
{
  char jpeg_header[5] = "\xff\xd8\xff\xe0";

  return !strncmp(header, jpeg_header, 4);

}


static struct jpeg_decompress_struct cinfo;
static struct my_error_mgr jerr;

static JSAMPARRAY buffer;		/* Output row buffer */
static int row_stride;		/* physical row width in output buffer */
static FILE *infile;

bool loadJpegInit( const char *filename, int *width, int *height)
{

  if ( (infile = fopen(filename, "rb") ) == NULL ){
    fprintf(stderr, "can't open %s\n", filename);
    return false;
  }

  cinfo.err = jpeg_std_error(&(jerr.pub));
  jerr.pub.error_exit = my_error_exit;

  if (setjmp(jerr.setjmp_buffer)) {
    jpeg_destroy_decompress(&(cinfo));
    return false;
  }

  jpeg_create_decompress(&cinfo);
  jpeg_stdio_src(&cinfo, infile);
  jpeg_read_header(&cinfo, TRUE);
  jpeg_start_decompress(&cinfo);

  if(cinfo.output_components == 1 || cinfo.output_components == 3){
    row_stride = cinfo.output_width * cinfo.output_components;
    buffer = cinfo.mem->alloc_sarray( (j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1 );
    
    *width  = cinfo.image_width;
    *height = cinfo.image_height;

    return true;
  }

  (void) jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);

  return false;

}


void makeJpeg( Image_Info *iinfo)
{
  int i, current_line, last_line;
  unsigned char *src;

  current_line = cinfo.output_scanline;  
  last_line = cinfo.output_height;

  while (current_line < last_line) {
    
    jpeg_read_scanlines(&cinfo, buffer, 1);
    current_line = cinfo.output_scanline;     

    src = (unsigned char*)(buffer[0]);
 
    switch ( cinfo.out_color_space ){
    case JCS_RGB:                 // RGB 24bit
      for( i = 0; i < iinfo->width; i++  ) {
     	*(iinfo->image)++ = *src++;
	*(iinfo->image)++ = *src++;
	*(iinfo->image)++ = *src++;
      }
      break;
    case JCS_GRAYSCALE:                 //gray scale 8bit
      for( i = 0; i < iinfo->width; i++ ) {
	*(iinfo->image)++ = *src;
	*(iinfo->image)++ = *src;
	*(iinfo->image)++ = *src++;
      }
      break;
    default:
      break;
    }

  }

  (void) jpeg_finish_decompress(&cinfo); 
  jpeg_destroy_decompress(&cinfo);
  
  fclose(infile);

}




void makeJpegS( short * dst, const int my_scale, Image_Info *iinfo)
{
  
  uint *line = new uint[iinfo->width * 3];    
  uint *lineData;
  unsigned char *src=0;
  int prev_line = 0;
  int lines = 0;
  int total_line = 0;
  
  int current_line = 0;
  int last_line = 0;

  current_line = cinfo.output_scanline;  
  last_line = cinfo.output_height;

  lineData = line;
  for(int i = 0; i < iinfo->width ; i++){
    *lineData++ = 0;
    *lineData++ = 0;
    *lineData++ = 0;
  }

  while (current_line < last_line) {
    
    jpeg_read_scanlines(&cinfo, buffer, 1);
    current_line = cinfo.output_scanline;     

    if(current_line < iinfo->start_v)
      continue;

    //
    // skip line
    //
    lineData = line;

    src = (unsigned char*)(buffer[0]);
 
    for(int i = 0; i < iinfo->width ; i++){
      *lineData++ += *src++;
      *lineData++ += *src++;
      *lineData++ += *src++;
    }
    lines++;
        
    if ( prev_line == ( (current_line-1) * my_scale)/100 )
      continue;
    
    //
    // draw line number
    //
    prev_line = ( (current_line-1) * my_scale)/100;

    total_line++;
    if (total_line > iinfo->dst_height)
      continue;
    //    if(current_line > (iinfo->src_height + iinfo->start_v - 1 ) )
    
    
    lineData = line;


    int r = 0, g = 0, b = 0;
    int i;
    int pixels = 0, prev_pixel = -1;
    short *last_pixel;
    last_pixel = dst + iinfo->dst_width; 

    switch ( cinfo.out_color_space ){
    case JCS_RGB:                 // RGB 24bit
      lineData += iinfo->start_h * 3;
      for( i = iinfo->start_h; i < iinfo->end_h; i++  ) {
	pixels++;
	r += *lineData++;
	g += *lineData++;
	b += *lineData++;
	if( prev_pixel == ((i * my_scale) / 100) )
	  continue;
	
	prev_pixel = ((i * my_scale) / 100);
	
	*dst++ = (( (r/(lines*pixels)) >> 3) << 11)|
	  (( (g/(lines*pixels)) >> 2) << 5) |
	  (( (b/(lines*pixels)) >> 3) << 0)  ;
	
	pixels = 0;
	r=0;g=0;b=0;
	if (dst >= last_pixel)
	  break;
      }
      break;
    case JCS_GRAYSCALE:                 //gray scale 8bit
      lineData += iinfo->start_h;
      for( i = iinfo->start_h; i < iinfo->end_h; i++ ) {
	pixels++;
	g += *lineData++;
	if( prev_pixel == ((i * my_scale) / 100) )
	  continue;
	
	prev_pixel = ((i * my_scale) / 100);
	*dst++ = (( (g/(lines*pixels)) >> 3) << 11)|
	  (( (g/(lines*pixels)) >> 2) << 5) |
	  (( (g/(lines*pixels)) >> 3) << 0)  ;
	
	pixels = 0;
	g = 0;
	if (dst >= last_pixel)
	  break;
      }
      break;
    default:
      break;
    }

    lineData = line;
    for(int i = 0; i < iinfo->width ; i++){
      *lineData++ = 0;
      *lineData++ = 0;
      *lineData++ = 0;
    }
    lines = 0;
  }
  delete line;
  
  
  (void) jpeg_finish_decompress(&cinfo); 
  jpeg_destroy_decompress(&cinfo);
  
  fclose(infile);

}
