/***************************************************
      load PNG routine

    Loading and scaling to scaledImage 16bit QImage

***************************************************/   
#include <sys/types.h> 
#include <unistd.h> 
#include <stdlib.h>
#include "loadpng.h"


#ifndef png_jmpbuf
#  define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
#endif

#define PNG_BYTES_TO_CHECK 4

static png_structp png_ptr;
static png_infop info_ptr;
static png_bytep buffer;
static int number_passes;
  
static FILE *infile;


int check_if_png(char *header)
{
  return(!png_sig_cmp((png_bytep)header, (png_size_t)0, PNG_BYTES_TO_CHECK));
}

bool loadPNGInit( const char *filename, int *width, int *height)
{
  unsigned int sig_read = 0;
  int bit_depth, color_type, interlace_type;

  if ( (infile = fopen(filename, "rb") ) == NULL ){
    fprintf(stderr, "can't open %s\n", filename);
    return false;
  }
  
  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  
  if (png_ptr == NULL){
    return false;
  }
  
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL){
    png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
    return false;
  }
  
  if (setjmp(png_jmpbuf(png_ptr))){
    png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
    return false;
  }
  
  png_init_io(png_ptr, infile);
  png_set_sig_bytes(png_ptr, sig_read);
  
  png_read_info(png_ptr, info_ptr);
  png_get_IHDR(png_ptr, info_ptr, (png_uint_32*)width, (png_uint_32*)height,
	       &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);
  number_passes = png_set_interlace_handling(png_ptr);
  png_set_strip_16(png_ptr);
  if (color_type == PNG_COLOR_TYPE_PALETTE)
    png_set_expand(png_ptr);
  png_set_strip_alpha(png_ptr);
  png_read_update_info(png_ptr, info_ptr);

  buffer = (png_bytep) png_malloc(png_ptr, png_get_rowbytes(png_ptr, info_ptr));

  return true;
}

void makePNG( Image_Info *iinfo)
{
  
  int pass, i, j;
  unsigned char *src;

  for(pass = 0; pass < number_passes; pass++){
    for ( i = 0; i < iinfo->height; i++) {
      png_read_row(png_ptr, buffer, png_bytep_NULL);
      src = (unsigned char*)(buffer);

      for( j = 0; j < iinfo->width; j++  ) {
	*(iinfo->image)++ = *src++;
	*(iinfo->image)++ = *src++;
	*(iinfo->image)++ = *src++;

      }
    }
  }
  png_read_end(png_ptr, info_ptr);

  free(buffer);

  png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);

  fclose(infile);

}
