#include "setnibble.h"

#include <qstring.h>
#include <qlabel.h>

#include <stdio.h>

SetNibbleDialog::SetNibbleDialog(BR_Option option, QWidget *parent)
  :QDialog (parent, "BackgroundColor", true )
{
  setCaption("Nibble Setting");

  setMinimumSize( 300, 100 );

  nibble_HBox = new QSpinBox( 1, 50, 1, this);
  nibble_VBox = new QSpinBox( 1, 50, 1, this);
  
  nibble_HBox->setValue( option.nib.divH );
  nibble_VBox->setValue( option.nib.divV );

  nibble_HBox->setGeometry( QRect(10, 30, 120, 30) );
  nibble_VBox->setGeometry( QRect(10, 90, 120, 30) );

  nibble_HBox->show();
  nibble_VBox->show();

  QLabel * HLabel = new QLabel( "Horiz Divider", this);
  QLabel * VLabel = new QLabel( "Vert Divider", this);

  HLabel->setGeometry( QRect(10, 0, 130, 20));
  VLabel->setGeometry( QRect(10, 70, 130, 20));
 
  HLabel->show();
  VLabel->show();


  
  nibble_LowHBox = new QSpinBox( 0, 100, 1, this);
  nibble_HighHBox = new QSpinBox( 0, 100, 1, this);
  nibble_LowVBox = new QSpinBox( 0, 100, 1, this);
  nibble_HighVBox = new QSpinBox( 0, 100, 1, this);
  
  nibble_LowHBox->setValue( option.nib.lowH );
  nibble_HighHBox->setValue( option.nib.highH );
  nibble_LowVBox->setValue( option.nib.lowV );
  nibble_HighVBox->setValue( option.nib.highV );

  nibble_LowHBox->setGeometry( QRect(100, 190, 70, 30) );
  nibble_HighHBox->setGeometry( QRect(100, 220, 70, 30) );
  nibble_LowVBox->setGeometry( QRect(250, 190, 70, 30) );
  nibble_HighVBox->setGeometry( QRect(250, 220, 70, 30) );

  nibble_LowHBox->show();
  nibble_HighHBox->show();
  nibble_LowVBox->show();
  nibble_HighVBox->show();

  QLabel * lowHLabel = new QLabel( "Low H %", this);
  QLabel * highHLabel = new QLabel( "High H %", this);
  QLabel * lowVLabel = new QLabel( "Low V %", this);
  QLabel * highVLabel = new QLabel( "High V %", this);

  lowHLabel->setGeometry( QRect(10, 190, 80, 20));
  highHLabel->setGeometry( QRect(10, 220, 80, 20));
  lowVLabel->setGeometry( QRect(170, 190, 80, 20));
  highVLabel->setGeometry( QRect(170, 220, 80, 20));
 
  lowHLabel->show();
  highHLabel->show();
  lowVLabel->show();
  highVLabel->show();



  nibLRCheck = new QRadioButton("Left mode",this);
  nibLRCheck->setGeometry( QRect(10, 120, 120, 30) );
  nibLRCheck->show();

  nibLRCheck->setChecked(option.nib.left);

  nibNovel = new QRadioButton("Novel mode",this);
  nibNovel->setGeometry( QRect(10, 150, 140, 30) );
  nibNovel->show();

  nibNovel->setChecked(option.nib.novel);

  nibButtons = new QButtonGroup(this);
  nibButtons->setGeometry( QRect(160, 20, 160, 160) );

  nibNone = new QRadioButton("None", nibButtons);
  nibNone->setGeometry( QRect(5, 5, 100, 30) );
  nibNone->show();

  nibHoriz = new QRadioButton("Horiz", nibButtons);
  nibHoriz->setGeometry( QRect(5, 35, 100, 30) );
  nibHoriz->show();

  nibVert = new QRadioButton("Vert", nibButtons);
  nibVert->setGeometry( QRect(5, 65, 100, 30) );
  nibVert->show();

  nibAutoHoriz = new QRadioButton("Auto Horiz", nibButtons);
  nibAutoHoriz->setGeometry( QRect(5, 95, 140, 30) );
  nibAutoHoriz->show();

  nibAutoVert = new QRadioButton("Auto Vert", nibButtons);
  nibAutoVert->setGeometry( QRect(5, 125, 140, 30) );
  nibAutoVert->show();


  QLabel * nibLabel = new QLabel( "Nibble Mode", this);
  nibLabel->setGeometry( QRect(170, 0, 120, 20));
  nibLabel->show();


  switch (option.nib.mode) {
  case NIB_Horiz:
    nibHoriz->setChecked(true);
    break;
  case NIB_Vert:
    nibVert->setChecked(true);
    break;
  case NIB_AutoHoriz:
    nibAutoHoriz->setChecked(true);
    break;
  case NIB_AutoVert:
    nibAutoVert->setChecked(true);
    break;
  case NIB_None:
  default:
    nibNone->setChecked(true);
    break;
  }


}

SetNibbleDialog::~SetNibbleDialog()
{
  // no need to delete child widgets, Qt does it all for us
}


NIB_Option SetNibbleDialog::getNIB_Opt()
{
  NIB_Option ret_opt;

  if(nibHoriz->isChecked() == true) {
    ret_opt.mode = NIB_Horiz;
  }else if(nibVert->isChecked() == true) {
    ret_opt.mode = NIB_Vert;
  }else  if(nibAutoHoriz->isChecked() == true){
    ret_opt.mode = NIB_AutoHoriz;
  }else if(nibAutoVert->isChecked() == true){
    ret_opt.mode = NIB_AutoVert;
  }else {
    ret_opt.mode = NIB_None;
  }

  ret_opt.divH = nibble_HBox->value();
  ret_opt.divV = nibble_VBox->value();

  ret_opt.left = nibLRCheck->isChecked();
  ret_opt.novel = nibNovel->isChecked();

  if ( nibble_LowHBox->value() < nibble_HighHBox->value() ){
    ret_opt.lowH = nibble_LowHBox->value();
    ret_opt.highH = nibble_HighHBox->value();
  }
  if ( nibble_LowVBox->value() < nibble_HighVBox->value() ){
    ret_opt.lowV = nibble_LowVBox->value();
    ret_opt.highV = nibble_HighVBox->value();
  }

  return ret_opt;

}
